@@ -74,7 +74,7 @@ CG_SNAPSHOT_MEMBERS_SORT_KEY_VALUES = (
 )
 
 EXPERIMENTAL_HTTP_HEADER = 'X-OpenStack-Manila-API-Experimental'
-MAX_API_VERSION = '2.6'
-V1_API_VERSION = '1.0'
 V1_SERVICE_TYPE = 'share'
 V2_SERVICE_TYPE = 'sharev2'
+
+SERVICE_TYPES = {'1': V1_SERVICE_TYPE, '2': V2_SERVICE_TYPE}
